#include <SFML/OpenGL.hpp>
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>
#include <string>
#include <iostream>
#include <ctime>
#include <cstdlib>
#include <tmx/MapLoader.h>
#include <sstream>
#include "Enemy.hpp"
#include "collision.h"
#include "FPS.hpp"
//globals(because they are needed in other classes...?(particles)
bool hitTruck = false;
bool hitEnemy = false;
bool truckPlaced = false;
bool canBePickedUp = false;
bool isPlayerDead = false;
bool firstPass = true;

#pragma region particles(constructor)
//particles
class ParticleSystem : public sf::Drawable, public sf::Transformable
{
public:

	ParticleSystem(unsigned int count) :
		m_particles(count),
		m_vertices(sf::Points, count),
		m_lifetime(sf::seconds(3)),
		m_emitter(0, 0),
		m_transparent(sf::Color(0, 0, 0, 255))    /*< sf::Color(0, 0, 0, 0) */
	{
	}

	void setEmitter(sf::Vector2f position)
	{
		m_emitter = position;
	}

	void update(sf::Time elapsed)
	{
		for (std::size_t i = 0; i < m_particles.size(); ++i)
		{
			// update the particle lifetime
			Particle& p = m_particles[i];
			p.lifetime -= elapsed;

			// if the particle is dead, respawn it
			if (p.lifetime <= sf::Time::Zero)
				resetParticle(i);

			// update the position of the corresponding vertex
			m_vertices[i].position += p.velocity * elapsed.asSeconds();

			// update the alpha (transparency) of the particle according to its lifetime
			float ratio = p.lifetime.asSeconds() / m_lifetime.asSeconds();
			m_vertices[i].color.a = static_cast<sf::Uint8>(ratio * 255);
			//update the color if the collisions are true
			if (truckPlaced){
				m_vertices[i].color.r = 255;
				m_vertices[i].color.g = 102;
				m_vertices[i].color.b = 0;
			}
			else if (!canBePickedUp)
			{
				m_vertices[i].color.r = 128;
				m_vertices[i].color.g = 0;
				m_vertices[i].color.b = 0;
			}
			else if (hitTruck){
				m_vertices[i].color.r = 0;
				m_vertices[i].color.g = 127;
				m_vertices[i].color.b = 255;
			}
			else if (hitEnemy)
			{
				m_vertices[i].color.r = 255;
				m_vertices[i].color.g = 255;
				m_vertices[i].color.b = 0;
			}
			else {
				m_vertices[i].color.r = 255;
				m_vertices[i].color.g = 255;
				m_vertices[i].color.b = 255;
			}
		}
	}

private:

	virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const
	{
		// apply the transform
		states.transform *= getTransform();

		// our particles don't use a texture
		states.texture = NULL;

		// draw the vertex array
		target.draw(m_vertices, states);
	}

private:

	struct Particle
	{
		sf::Vector2f velocity;
		sf::Time lifetime;
	};

	void resetParticle(std::size_t index)
	{
		if (hitTruck || hitEnemy){
			float angle = (std::rand() % 360) * 3.14f / 180.f;
			float speed = (std::rand() % 15) + 15.f;//was 50 for both
			m_particles[index].velocity = sf::Vector2f(std::cos(angle) * speed, std::sin(angle) * speed);
			m_particles[index].lifetime = sf::milliseconds((std::rand() % 2000) + 1000);//was % 2000) + 1000)
		}
		else{
		// give a random velocity and lifetime to the particle
		float angle = (std::rand() % 360) * 3.14f / 180.f;
		float speed = (std::rand() % 5) + 5.f;//was 50 for both
		m_particles[index].velocity = sf::Vector2f(std::cos(angle) * speed, std::sin(angle) * speed);
		m_particles[index].lifetime = sf::milliseconds((std::rand() % 2000) + 2000);//was % 2000) + 1000)
		}
		// reset the position of the corresponding vertex
		m_vertices[index].position = m_emitter;
	}

	std::vector<Particle> m_particles;
	sf::VertexArray m_vertices;
	sf::Time m_lifetime;
	sf::Vector2f m_emitter;
	sf::Color m_transparent;    /*< sf::Color(0, 0, 0, 0) */
};
#pragma endregion

int main()
{
	sf::Vector2i sDimensions(1440, 900);
	sf::Vector2i bDimensions(10, 10);
	enum Direction{ Up, Down, Left, Right, PURightAndPULeftCWalkLeft, PUDownPUUpCWalkUp, CWalkRightCWalkDown };
	enum Direction2{ Up2, Down2, Left2, Right2 };
	sf::Vector2i source(1, Up);
	sf::Vector2i source2(1, Up2);
	sf::RenderWindow window(sf::VideoMode(sDimensions.x, sDimensions.y), "Ursplaticus");
	window.setMouseCursorVisible(false);//hide cursor

	srand(time(0));

	ParticleSystem particles(16000);///////////////////////////////////////////////////////THE PARCILES ARE HERE NOTICE MEEEEEE

	sf::Texture truckTexture;
	sf::Texture pTexture;
	sf::Texture eTexture;
	sf::Sprite truckImage;
	sf::Sprite playerImage;
	sf::Sprite enemyImage;
	sf::Clock clock;
	sf::Clock clock2;
	sf::Clock particleClock;
	sf::Clock hotClock;

	sf::Clock FPSClock;

	sf::Time taken;
	//setup fonts
	sf::Font myfont;
	if (!myfont.loadFromFile("fonts/Ubuntu-M.ttf"))
	{
		std::cout << "font not loaded" << std::endl;
		//do nothing its just a test
	}
	int score = 0;
	sf::Text text("Score: ", myfont);
	text.setFont(myfont);
	text.setPosition(10, 10);
	text.setColor(sf::Color(255, 0, 255, 255));
	text.setScale(1, 1);//not sure about this.

	sf::Text deathtext("Aw you died!! Your final score was: ", myfont);
	deathtext.setFont(myfont);
	deathtext.setColor(sf::Color(255, 0, 255, 255));
	deathtext.setScale(1, 1);//not sure about this.

#pragma region sounds

	//sounds
	sf::SoundBuffer soundBuffer;
	sf::Sound sfx_pickup;
	if (!soundBuffer.loadFromFile("media/pickup.wav"))
		std::cout << "can't find pickup.wav" << std::endl;
	sfx_pickup.setBuffer(soundBuffer);
	sfx_pickup.setPitch(1.0f);//default is 1. lower = duller pitch.
	sfx_pickup.setVolume(50);//defaults at 100
	sfx_pickup.setLoop(false);//true loops.

	sf::SoundBuffer soundBuffer2;
	sf::Sound sfx_ploik;
	if (!soundBuffer2.loadFromFile("media/ploik.wav"))
		std::cout << "can't find pickup.wav" << std::endl;
	sfx_ploik.setBuffer(soundBuffer2);
	sfx_ploik.setPitch(.9f);//default is 1. lower = duller pitch.
	sfx_ploik.setVolume(50);//defaults at 100
	sfx_ploik.setLoop(false);//true loops.

	sf::SoundBuffer soundBuffer3;
	sf::Sound sfx_splat;
	if (!soundBuffer3.loadFromFile("media/splat.wav"))
		std::cout << "can't find pickup.wav" << std::endl;
	sfx_splat.setBuffer(soundBuffer3);
	sfx_splat.setPitch(1.0f);//default is 1. lower = duller pitch.
	sfx_splat.setVolume(50);//defaults at 100
	sfx_splat.setLoop(false);//true loops.

	sf::SoundBuffer soundBuffer4;
	sf::Sound sfx_drop;
	if (!soundBuffer4.loadFromFile("media/drop.wav"))
		std::cout << "can't find drop.wav" << std::endl;
	sfx_drop.setBuffer(soundBuffer4);
	sfx_drop.setPitch(.7f);//default is 1. lower = duller pitch.
	sfx_drop.setVolume(50);//defaults at 100
	sfx_drop.setLoop(false);//true loops.

	sf::SoundBuffer soundBuffer5;
	sf::Sound sfx_laugh;
	if (!soundBuffer5.loadFromFile("media/laugh.wav"))
		std::cout << "can't find drop.wav" << std::endl;
	sfx_laugh.setBuffer(soundBuffer5);
	sfx_laugh.setPitch(1.0f);//default is 1. lower = duller pitch.
	sfx_laugh.setVolume(50);//defaults at 100
	sfx_laugh.setLoop(false);//true loops.
	 
#pragma endregion

	float PSPEED = 1;//was using 10 for testing
	float eSpeed = .2;//was using 10 for testing
	float frameCounter = 0, switchFrame = 100, frameSpeed = 800;
	float frameCounter2 = 0, switchFrame2 = 100, frameSpeed2 = 800;

	sf::RectangleShape rect(sf::Vector2f(20, 60));
	rect.setFillColor(sf::Color(255, 0, 0, 100));
	rect.setPosition(785, 680);
	rect.setOutlineColor(sf::Color(0, 0, 0, 100));
	rect.setOutlineThickness(2);

#pragma region bools
	bool updateFrame = false;
	bool updateFrame2 = false;
	bool frameStart = false;
	bool frameStartPicking = false; //set start animation frame
	bool isPickingUpTruck = false; //is the player picking up the truck?
	bool hasTruck = false; //is the player holding the truck?
	bool theOne = true; //only one animation at a time thank you.
	bool UU = false;//which animation do i need for which way i'm facing. (it's because i left the animations all on the same line on the sprite sheet
	bool DD = true;
	bool LL = false;
	bool RR = false;
#pragma endregion

	if (!pTexture.loadFromFile("media/player.png"))
		std::cout << "could not load my player texture" << std::endl;
	if (!eTexture.loadFromFile("media/enemy.png"))
		std::cout << "could not load my enemy texture" << std::endl;
	if (!truckTexture.loadFromFile("media/truck.png"))
		std::cout << "could not load my truck texture" << std::endl;
	playerImage.setTexture(pTexture);
	playerImage.setPosition(900, 680);

	truckImage.setTexture(truckTexture);
	truckImage.setPosition(600, 480);
	enemyImage.setTexture(eTexture);
	enemyImage.setPosition(1000, 1000);//while this is useful, not what i need till we kill the thingy: std::rand() % sDimensions.x, std::rand() % sDimensions.y);//randomly place the person anywhere on the map.

	tmx::MapLoader ml("media/");
	ml.Load("tilemapmain.tmx");

	//view!
	sf::View view;
	view.reset(sf::FloatRect(0, 0, sDimensions.x, sDimensions.y));
	view.setViewport(sf::FloatRect(0, 0, 1.5f, 1.5f));
	//view.setSize(720, 480);

	sf::Vector2f position(sDimensions.x / 2, sDimensions.y / 2);

	while (window.isOpen())
	{

#pragma region eventloop
		sf::Event event;
		while (window.pollEvent(event))
		{
			switch (event.type)
			{
			case sf::Event::Closed:
				window.close();
				break;
			case sf::Event::KeyPressed:
				if (event.key.code == sf::Keyboard::Escape)
					window.close();
				if (event.key.code == sf::Keyboard::W || event.key.code == sf::Keyboard::Up)
				{
					UU = true; LL = false; DD = false; RR = false;
				}
				else if (event.key.code == sf::Keyboard::A || event.key.code == sf::Keyboard::Left)
				{
					LL = true; UU = false; DD = false; RR = false;
				}
				else if (event.key.code == sf::Keyboard::S || event.key.code == sf::Keyboard::Down)
				{
					DD = true; UU = false; LL = false; RR = false;
				}
				if (event.key.code == sf::Keyboard::D || event.key.code == sf::Keyboard::Right)
				{
					RR = true; UU = false; LL = false; DD = false;
				}
				break;
			case sf::Event::GainedFocus:
				std::cout << "window active" << std::endl;
				break;
			case sf::Event::LostFocus:
				std::cout << "window not active!" << std::endl;
				break;
			case sf::Event::KeyReleased://when we release the movement keys, reset the animation to "0"
				if (event.key.code == sf::Keyboard::Up || event.key.code == sf::Keyboard::Down || event.key.code == sf::Keyboard::Left || event.key.code == sf::Keyboard::Right ||
					event.key.code == sf::Keyboard::W || event.key.code == sf::Keyboard::S || event.key.code == sf::Keyboard::A || event.key.code == sf::Keyboard::D)
					{
						if (isPickingUpTruck){
							frameStartPicking = true;
						}
						else if (hasTruck)
						{
							frameStart = true;
							//i neeed it to change to the right START at teh START of the animation withou interrupting the frame update.
							if (UU){
								updateFrame = false;
								source.x = 5;
							}
							if (DD){
								updateFrame = false;
								source.x = 3;
							}
							if (RR){
								updateFrame = false;
								source.x = 0;
							}
							if (LL){
								updateFrame = false;
								source.x = 5;
							}
						}
						else
						{
							updateFrame = false;
							source.x = 0;
						}
					}
			}
		}
#pragma endregion

		//make the particles follow the mouse
		sf::Vector2i mouse = sf::Mouse::getPosition(window);
		particles.setEmitter(window.mapPixelToCoords(mouse));
		//update particles
		sf::Time elapsed = particleClock.restart();
		particles.update(elapsed);
		std::string i_str = std::to_string(score);
		text.setString("Score: " + i_str);

#pragma region enemy_movement
		//enemy movement
		if (enemyImage.getPosition().x > playerImage.getPosition().x)//left
		{
			updateFrame2 = true;
			source2.y = Left2;
			enemyImage.move(-eSpeed, 0);
		}
		if (enemyImage.getPosition().x <= playerImage.getPosition().x)//right
		{
			updateFrame2 = true;
			source2.y = Right2;
			enemyImage.move(eSpeed, 0);
		}
		if (enemyImage.getPosition().y > playerImage.getPosition().y)//up
		{
			updateFrame2 = true;
			source2.y = Up2;
			enemyImage.move(0, -eSpeed);
		}
		if (enemyImage.getPosition().y <= playerImage.getPosition().y)//down
		{
			updateFrame2 = true;
			source2.y = Down2;
			enemyImage.move(0, eSpeed);
		}
		if (updateFrame2)//this is so the animation can stop when you don't want it going.
			frameCounter2 += frameSpeed2 * clock2.restart().asSeconds();
		if (frameCounter2 >= switchFrame2)
		{
			frameCounter2 = 0;
			source2.x++;
			if (source2.y == Right){
				if (source2.x * 46 >= 5 * 46)//frame i want the animation to end.
					source2.x = 1;
			}
			else if (source2.y == Left){
				if (source2.x * 46 >= 5 * 46)//frame i want the animation to end.
					source2.x = 1;
			}
			if (source2.y == Up){
				if (source2.x * 46 >= 5 * 46)//frame i want the animation to end.
					source2.x = 1;
			}
			else if (source2.y == Down){
				if (source2.x * 46 >= 5 * 46)//frame i want the animation to end.
					source2.x = 1;
			}
			else if (source2.x * 46 >= eTexture.getSize().x)
				source2.x = 1;
		}
#pragma endregion

#pragma region player_movement
		//player movement
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Up) || sf::Keyboard::isKeyPressed(sf::Keyboard::W))
		{
			updateFrame = true;//PURightAndPULeftCWalkLeft, PUDownPUUpCWalkUp, CWalkRightCWalkDown 
			if (isPickingUpTruck)
			{
			}
			else if (hasTruck){
				source.y = PUDownPUUpCWalkUp;
				playerImage.move(0, -PSPEED + .1);
				frameSpeed = 800;
			}
			else
			{
				source.y = Up;
				playerImage.move(0, -PSPEED);//for now all the animations use the same speed and framespeed.
				frameSpeed = 1200;
			}
		}
		else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Down) || sf::Keyboard::isKeyPressed(sf::Keyboard::S))
		{
			updateFrame = true;
			if (isPickingUpTruck)
			{
			}
			else if (hasTruck)
			{
				source.y = CWalkRightCWalkDown;
				playerImage.move(0, PSPEED - .1);
				frameSpeed = 800;
			}
			else
			{
				source.y = Down;
				playerImage.move(0, PSPEED);
				frameSpeed = 1200;
			}
		}
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Right) || sf::Keyboard::isKeyPressed(sf::Keyboard::D))
		{
			updateFrame = true;
			if (isPickingUpTruck)
			{
			}
			else if (hasTruck)
			{
				source.y = CWalkRightCWalkDown;
				playerImage.move(PSPEED - .1, 0);
				frameSpeed = 800;
			}
			else
			{
				source.y = Right;
				playerImage.move(PSPEED, 0);
				frameSpeed = 800;
			}
		}
		else if (sf::Keyboard::isKeyPressed(sf::Keyboard::Left) || sf::Keyboard::isKeyPressed(sf::Keyboard::A))
		{
			updateFrame = true;
			if (isPickingUpTruck){
			}
			else if (hasTruck){
				source.y = PURightAndPULeftCWalkLeft;
				playerImage.move(-PSPEED + .1, 0);
				frameSpeed = 800;
			}
			else
			{
				source.y = Left;
				playerImage.move(-PSPEED, 0);
				frameSpeed = 800;
			}
		}
		//this is for setting the animation source of the frame to be correct when you push the button
		if (hasTruck && frameStart && (sf::Keyboard::isKeyPressed(sf::Keyboard::Up) || sf::Keyboard::isKeyPressed(sf::Keyboard::W) || sf::Keyboard::isKeyPressed(sf::Keyboard::Down) || 
			sf::Keyboard::isKeyPressed(sf::Keyboard::S) || sf::Keyboard::isKeyPressed(sf::Keyboard::Right) || sf::Keyboard::isKeyPressed(sf::Keyboard::D) || 
			sf::Keyboard::isKeyPressed(sf::Keyboard::Left) || sf::Keyboard::isKeyPressed(sf::Keyboard::A)))
		{
			frameStart = false;
			//i neeed it to change to the right START at teh START of the animation withou interrupting the frame update.
			if (UU){
				//updateFrame = true;
				source.x = 5;
			}
			if (DD){
				//updateFrame = true;
				source.x = 3;
			}
			if (RR){
				//updateFrame = true;
				source.x = 0;
			}
			if (LL){
				//updateFrame = true;
				source.x = 5;
			}
		}
		if (isPickingUpTruck && frameStartPicking && (sf::Keyboard::isKeyPressed(sf::Keyboard::Up) || sf::Keyboard::isKeyPressed(sf::Keyboard::W) || sf::Keyboard::isKeyPressed(sf::Keyboard::Down) ||
			sf::Keyboard::isKeyPressed(sf::Keyboard::S) || sf::Keyboard::isKeyPressed(sf::Keyboard::Right) || sf::Keyboard::isKeyPressed(sf::Keyboard::D) ||
			sf::Keyboard::isKeyPressed(sf::Keyboard::Left) || sf::Keyboard::isKeyPressed(sf::Keyboard::A) || sf::Mouse::isButtonPressed(sf::Mouse::Left)))
		{
			frameStartPicking = false;
			//i neeed it to change to the right START at teh START of the animation withou interrupting the frame update.
			if (UU){
				updateFrame = true;
				source.x = 3;
			}
			if (DD){
				updateFrame = true;
				source.x = 0;
			}
			if (RR){
				updateFrame = true;
				source.x = 0;
			}
			if (LL){
				updateFrame = true;
				source.x = 3;
			}
		}
		//these need moved out so that they play and are not dependent on whether you are clicking the button or not.
		if (UU && isPickingUpTruck && theOne){
			theOne = false;
			source.y = PUDownPUUpCWalkUp;
			playerImage.move(0, -.1);
			frameSpeed = 1300;
		}
		if (DD  && isPickingUpTruck && theOne){
			theOne = false;
			source.y = PUDownPUUpCWalkUp;
			playerImage.move(0, .1);
			frameSpeed = 1300;
		}
		if (LL  && isPickingUpTruck && theOne){
			theOne = false; 
			source.y = PURightAndPULeftCWalkLeft;
			playerImage.move(-.1, 0);
			frameSpeed = 1300;
		}
		if (RR  && isPickingUpTruck && theOne){
			theOne = false;
			source.y = PURightAndPULeftCWalkLeft;
			playerImage.move(.1, 0);
			frameSpeed = 1300;
		}
		//should animate the player...
		/*if (sf::Mouse::isButtonPressed(sf::Mouse::Left))
			updateFrame = true;
		else if (sf::Mouse::isButtonPressed(sf::Mouse::Right))
			updateFrame = false;*/
		if (updateFrame)//this is so the animation can stop when you don't want it going.
			frameCounter += frameSpeed * clock.restart().asSeconds();
		if (frameCounter >= switchFrame)
		{
			frameCounter = 0;
			source.x++;
			if (source.y == Right){
				if (source.x * 46 >= 5 * 46)//frame i want the animation to end. 1, 2, 3, 4;
					source.x = 1;//frame to cyle to when it's done. 
			}
			else if (source.y == Left){
				if (source.x * 46 >= 5 * 46)//frame i want the animation to end.
					source.x = 1;
			}
			if (source.y == Up){
				if (source.x * 46 >= 5 * 46)//frame i want the animation to end.
					source.x = 1;
			}
			else if (source.y == Down){
				if (source.x * 46 >= 5 * 46)//frame i want the animation to end.
					source.x = 1;
			}
			if (source.y == PUDownPUUpCWalkUp && isPickingUpTruck && UU){//PUUp 
				if (source.x * 46 >= 6 * 46)//frame i want the animation to end.//test this testing 3,4,5
				{
					theOne = true;
					isPickingUpTruck = false;
					hasTruck = true;
					source.x = 3;
					updateFrame = false;
				}
			}
			else if (source.y == PUDownPUUpCWalkUp && isPickingUpTruck && DD){//PUdown 
				if (source.x * 46 >= 3 * 46)//frame i want the animation to end.0,1,2
				{
					theOne = true;
					isPickingUpTruck = false;
					hasTruck = true;
					source.x = 0;
					updateFrame = false;
				}
			}
			if (source.y == PURightAndPULeftCWalkLeft && isPickingUpTruck && RR){//PURight
				if (source.x * 46 >= 3 * 46)//frame i want the animation to end.0,1,2
				{
					theOne = true;
					isPickingUpTruck = false;
					hasTruck = true;
					source.x = 0;
					updateFrame = false;
				}
			}
			else if (source.y == PURightAndPULeftCWalkLeft && isPickingUpTruck && LL){//PULeft
				if (source.x * 46 >= 6 * 46)//frame i want the animation to end.3,4,5
				{
					theOne = true;
					isPickingUpTruck = false;
					hasTruck = true;
					source.x = 3;
					updateFrame = false;
				}
			}
			if (source.y == PUDownPUUpCWalkUp && hasTruck){//CWalkUp 
				if (source.x * 46 >= 8 * 46)//frame i want the animation to end.5,6,7
					source.x = 5;
			}
			else if (source.y == CWalkRightCWalkDown && hasTruck && DD){//CWalkDown 
				if (source.x * 46 >= 6 * 46)//frame i want the animation to end.3,4,5
					source.x = 3;
			}
			if (source.y == CWalkRightCWalkDown && hasTruck && RR){//CWalkRight 
				if (source.x * 46 >= 3 * 46)//frame i want the animation to end.0,1,2
					source.x = 0;
			}
			else if (source.y == PURightAndPULeftCWalkLeft && hasTruck){//CWalkLeft
				if (source.x * 46 >= 8 * 46)//frame i want the animation to end.5,6,7
					source.x = 5;
			}
			//else if (source.x * 46 >= pTexture.getSize().x)
				//source.x = 1;
		}
#pragma endregion

#pragma region collisions
		if (Collision::PixelPerfectTest(enemyImage, playerImage, 0))
		{
			if (!firstPass){
				updateFrame2 = false;
				source2.x = 0;
				isPlayerDead = true;//player dies here.
				PSPEED = 0;
			}
		}
		//place the truck. start the timer. check to see if timer is greater than .5. if timer is  greater than point 5. truckplaced is false. reset clock.
		if (truckPlaced){
			taken = hotClock.getElapsedTime();
			//std::cout << taken.asSeconds() << std::endl;
		}
		if (taken.asSeconds() >= .5){
			truckPlaced = false;
			taken = hotClock.restart();
		}
		if (Collision::PixelPerfectTest(enemyImage, truckImage, 0) && truckPlaced)//the truck has been placed. for .5 secs.
		{
			score++;
			updateFrame2 = false;
			source2.x = 0;
			sfx_splat.play();
			sfx_ploik.play();
			enemyImage.setPosition(std::rand() % sDimensions.x, std::rand() % sDimensions.y);//go to another random place on the map? yeah?
			std::cout << "OW!" << std::endl;
			eSpeed+=.1;
		}
		//sf::Vector2i mousePosition = sf::Mouse::getPosition();//global monitor relative to this monitor position (at least thats what it seems like)
		sf::Vector2i mousePosition = sf::Mouse::getPosition(window);//window relative position (the INT way of getting the position of the mouse)
		sf::FloatRect boundingBox = truckImage.getGlobalBounds();
		sf::FloatRect boundingBoxEnemy = enemyImage.getGlobalBounds();
		if (boundingBox.contains(window.mapPixelToCoords(mouse)))//the only "real" way to get the float position of the mouse. bother!
		{
			hitTruck = true;//for the particle animation
			if (sf::Mouse::isButtonPressed(sf::Mouse::Left) && !hasTruck && !isPickingUpTruck && canBePickedUp){
				canBePickedUp = false;
				isPickingUpTruck = true;
				//reset the animation for isPickingUpTruck!!
				sfx_pickup.play();
				updateFrame = true;
				if (UU)
					source.x = 3;
				else if (DD)
					source.x = 0;
				else if (LL)
					source.x = 3;
				else if (RR)
					source.x = 0;
				//run the animation to move the truck
				//when it's done, then run the truck holding animation.
			}
		}
		else if (boundingBoxEnemy.contains(window.mapPixelToCoords(mouse)))
		{//for particle animations
			hitEnemy = true;
			if (sf::Mouse::isButtonPressed(sf::Mouse::Left))//this is to testing the sound splat
				sfx_laugh.play();
		}
		else
		{
			canBePickedUp = true;//not sure what this does tesing <--
			hitTruck = false;
			hitEnemy = false;
		}
#pragma endregion 

		if (!isPickingUpTruck && hasTruck && sf::Mouse::isButtonPressed(sf::Mouse::Left)){
			//move the truck here
			taken = hotClock.restart();
			truckPlaced = true;
			canBePickedUp = false;
			hasTruck = false;
			truckImage.setPosition(window.mapPixelToCoords(mouse));
			truckImage.move(-100, -100);
			sfx_drop.play();
		}
		if (hasTruck){
			float pIx = playerImage.getPosition().x - 160;
			float pIy = playerImage.getPosition().y - 170;
			truckImage.setPosition(pIx, pIy);
		}
		//sf::Mouse::setPosition(sf::Vector2i(100, 100), window);//set mouse position relative to the game window.
		//center camra on player position if not near the edge of the map
		position.x = playerImage.getPosition().x - 20 - (sDimensions.x / 4);
		position.y = playerImage.getPosition().y - 20 - (sDimensions.y / 4);
		if (position.x < 0)
			position.x = 0;
		if (position.y < 0)
			position.y = 0;

		view.reset(sf::FloatRect(position.x, position.y, sDimensions.x, sDimensions.y));
		//set player texture
		playerImage.setTextureRect(sf::IntRect(source.x * 46, source.y * 56, 46, 56));
		enemyImage.setTextureRect(sf::IntRect(source2.x * 46, source2.y * 56, 46, 56));

		if (!isPlayerDead)
		{
			//setView
			window.setView(view);
			//draw
			window.draw(ml);
			window.draw(rect);
			window.draw(playerImage);
			window.draw(enemyImage);
			window.draw(truckImage);
			window.draw(particles);
			//set default view to draw the hud
			window.setView(window.getDefaultView());
			window.draw(text);
			window.setView(view);//and set it back again because mouse issues.
		}
		else
		{
			std::string i_str = std::to_string(score);
			deathtext.setPosition(playerImage.getPosition().x-200, playerImage.getPosition().y-50);
			deathtext.setString("Aw, you died. Your final score was: " + i_str + "\n Click esc to exit");
			window.setView(window.getDefaultView());
			window.draw(deathtext);
		}
		//display
		window.display();
		//clear
		window.clear();
		firstPass = false;

	}

}